@@ -114,6 +114,14 @@ paned  = wTree.get_widget('pan-main')
 window = wTree.get_widget('win-main')
 prefs.setWidgetsTree(wTree)
 
+# RGBA support
+try:
+    colormap = window.get_screen().get_rgba_colormap()
+    if colormap:
+        gtk.widget_set_default_colormap(colormap)
+except:
+    log.logger.info('No RGBA support (requires PyGTK 2.10+)')
+
 # Restore sizes
 window.resize(prefs.get(__name__, 'win-width', 750), prefs.get(__name__, 'win-height', 470))
 paned.set_position(prefs.get(__name__, 'paned-pos', 300))
