@@ -16,7 +16,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 #
-# ExtListView v1.5
+# ExtListView v1.7
+#
+# v1.7:
+#   * (Hopefully) fixed the intermittent improper columns resizing
+#
+# v1.6:
+#   * Added a context menu to column headers allowing users to show/hide columns
+#   * Improved sorting a bit
 #
 # v1.5:
 #   * Fixed intermittent improper columns resizing
@@ -46,7 +53,7 @@
 import gtk, random
 
 from gtk     import gdk
-from gobject import signal_new, TYPE_INT, TYPE_BOOLEAN, TYPE_PYOBJECT, TYPE_NONE, SIGNAL_RUN_LAST
+from gobject import signal_new, TYPE_INT, TYPE_STRING, TYPE_BOOLEAN, TYPE_PYOBJECT, TYPE_NONE, SIGNAL_RUN_LAST
 
 
 # Internal d'n'd (reordering)
@@ -58,14 +65,45 @@ DND_INTERNAL_TARGET = ('extListview-internal', gtk.TARGET_SAME_WIDGET, DND_REORD
 signal_new('extlistview-dnd', gtk.TreeView, SIGNAL_RUN_LAST, TYPE_NONE, (gdk.DragContext, TYPE_INT, TYPE_INT, gtk.SelectionData, TYPE_INT, TYPE_PYOBJECT))
 signal_new('extlistview-modified', gtk.TreeView, SIGNAL_RUN_LAST, TYPE_NONE, ())
 signal_new('extlistview-button-pressed', gtk.TreeView, SIGNAL_RUN_LAST, TYPE_NONE, (gdk.Event, TYPE_PYOBJECT))
+signal_new('extlistview-column-visibility-changed', gtk.TreeView, SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING, TYPE_BOOLEAN))
+signal_new('button-press-event', gtk.TreeViewColumn, SIGNAL_RUN_LAST, TYPE_NONE, (gdk.Event, ))
+
+
+class ExtListViewColumn(gtk.TreeViewColumn):
+    """
+        TreeViewColumn does not signal right-click events, and we need them
+        This subclass is equivalent to TreeViewColumn, but it signals these events
+
+        Most of the code in this class comes from Quod Libet (http://www.sacredchao.net/quodlibet)
+    """
+
+    def __init__(self, title=None, cell_renderer=None, **args):
+        """ Constructor, see gtk.TreeViewColumn """
+        gtk.TreeViewColumn.__init__(self, title, cell_renderer, **args)
+        label = gtk.Label(title)
+        self.set_widget(label)
+        label.show()
+        label.__realize = label.connect('realize', self.onRealize)
+
+
+    def onRealize(self, widget):
+        widget.disconnect(widget.__realize)
+        del widget.__realize
+        button = widget.get_ancestor(gtk.Button)
+        if button is not None:
+            button.connect('button-press-event', self.onButtonPressed)
+
+
+    def onButtonPressed(self, widget, event):
+        self.emit('button-press-event', event)
 
 
 class ExtListView(gtk.TreeView):
 
 
-    def __init__(self, columns, sortable=True, dndTargets=[], useMarkup=False):
+    def __init__(self, columns, sortable=True, dndTargets=[], useMarkup=False, canShowHideColumns=True):
         """
-            If sortable is True, the user can click on headers to sort the content of the list
+            If sortable is True, the user can click on headers to sort the contents of the list
 
             The d'n'd targets are the targets accepted by the list (e.g., [('text/uri-list', 0, 0)])
             Note that for the latter, the identifier 1024 must not be used (internally used for reordering)
@@ -89,14 +127,17 @@ class ExtListView(gtk.TreeView):
         # Create the columns
         nbEntries = 0
         dataTypes = []
-        for (title, renderers, sortIndexes, expandable) in columns:
+        for (title, renderers, sortIndexes, expandable, visible) in columns:
             if title is None:
                 nbEntries += len(renderers)
                 dataTypes += [renderer[1] for renderer in renderers]
             else:
-                column = gtk.TreeViewColumn(title)
+                column = ExtListViewColumn(title)
                 column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
                 column.set_expand(expandable)
+                column.set_visible(visible)
+                if canShowHideColumns:
+                    column.connect('button-press-event', self.onColumnHeaderClicked)
                 self.append_column(column)
 
                 if sortable:
@@ -151,6 +192,12 @@ class ExtListView(gtk.TreeView):
         return [self.store.get_iter(path) for path in self.selection.get_selected_rows()[1]]
 
 
+    def __resizeColumns(self):
+        """ That's the only way I could find to make sure columns are correctly resized (e.g., columns_autosize() has no effect) """
+        for column in self.get_columns():
+            column.queue_resize()
+
+
     def addColumnAttribute(self, colIndex, renderer, attribute, value):
         """ Add a new attribute to the given column """
         self.get_column(colIndex).add_attribute(renderer, attribute, value)
@@ -216,10 +263,17 @@ class ExtListView(gtk.TreeView):
 
     def __cmpRows(self, row1, row2, criteria, ascending):
         """ Compare two rows based on the given criteria, the latter being a tuple of the indexes to use for the comparison """
-        for index, criterion in enumerate(criteria):
+        # Sorting on the first criterion may be done either ascending or descending
+        criterion = criteria[0]
+        result    = cmp(row1[criterion], row2[criterion])
 
-            if ascending or index != 0: result = cmp(row1[criterion], row2[criterion])
-            else:                       result = cmp(row2[criterion], row1[criterion])
+        if result != 0:
+            if ascending: return result
+            else:         return -result
+
+        # For subsequent criteria, the order is always ascending
+        for criterion in criteria[1:]:
+            result = cmp(row1[criterion], row2[criterion])
 
             if result != 0:
                 return result
@@ -235,14 +289,14 @@ class ExtListView(gtk.TreeView):
         if self.sortLastCol is not None:
             self.sortLastCol.set_sort_indicator(False)
 
-        # Find how sorting must be performed
+        # Find how sorting must be done
         if self.sortLastCol == column:
             self.sortAscending = not self.sortAscending
         else:
             self.sortLastCol   = column
             self.sortAscending = True
 
-        # Dump the rows, sort them and reorder the list
+        # Dump the rows, sort them, and reorder the list
         rows     = [tuple(r) + (i,) for i, r in enumerate(self.store)]
         criteria = self.sortColCriteria[column]
         rows.sort(lambda r1, r2: self.__cmpRows(r1, r2, criteria, self.sortAscending))
@@ -340,8 +394,7 @@ class ExtListView(gtk.TreeView):
         self.__resetSorting()
         self.clearMark()
         self.store.clear()
-        # This fixes the problem of columns sometimes not resizing correctly
-        self.resize_children()
+        self.__resizeColumns()
 
 
     def setItem(self, rowIndex, colIndex, value):
@@ -368,8 +421,7 @@ class ExtListView(gtk.TreeView):
         if len(self.store) == 0:
             self.set_cursor(0)
             self.__resetSorting()
-        # This fixes the problem of columns sometimes not resizing correctly
-        self.resize_children()
+        self.__resizeColumns()
         self.emit('extlistview-modified')
 
 
@@ -390,7 +442,7 @@ class ExtListView(gtk.TreeView):
         if len(rows) == 0:
             return
 
-        # Insert the additional column, used for the mark management
+        # Insert the additional column used for the mark management
         if type(rows[0]) is tuple: rows[:] = [row + (False,) for row in rows]
         else:                      rows[:] = [row + [False] for row in rows]
 
@@ -573,3 +625,36 @@ class ExtListView(gtk.TreeView):
             self.enable_model_drag_dest([('invalid-position', 0, -1)], gdk.ACTION_DEFAULT)
         else:
             self.enable_model_drag_dest(self.dndTargets, gdk.ACTION_DEFAULT)
+
+
+    def onColumnHeaderClicked(self, column, event):
+        """ A column header has been clicked """
+        if event.button == 3:
+            # Create a menu with a CheckMenuItem per column
+            menu            = gtk.Menu()
+            nbVisibleItems  = 0
+            lastVisibleItem = None
+            for column in self.get_columns():
+                item = gtk.CheckMenuItem(column.get_title())
+                item.set_active(column.get_visible())
+                item.connect('toggled', self.onShowHideColumn, column)
+                item.show()
+                menu.append(item)
+
+                # Count how many columns are visible
+                if item.get_active():
+                    nbVisibleItems  += 1
+                    lastVisibleItem  = item
+
+            # Don't allow the user to hide the only visible column left
+            if nbVisibleItems == 1:
+                lastVisibleItem.set_sensitive(False)
+
+            menu.popup(None, None, None, event.button, event.get_time())
+
+
+    def onShowHideColumn(self, menuItem, column):
+        """ Switch the visibility of the given column """
+        column.set_visible(not column.get_visible())
+        self.__resizeColumns()
+        self.emit('extlistview-column-visibility-changed', column.get_title(), column.get_visible())
