@@ -122,6 +122,11 @@ class ExtTreeView(gtk.TreeView):
         return tuple(self.store[path])
 
 
+    def getRows(self, paths):
+        """ Return the given rows """
+        return [tuple(self.store[path]) for path in paths]
+
+
     def getSelectedRows(self):
         """ Return selected row(s) """
         return [tuple(self.store[path]) for path in self.selection.get_selected_rows()[1]]
