@@ -87,15 +87,26 @@ class AudioCD(modules.ThreadedModule):
             self.clearCache()
 
 
-    def getTracksFromSelectedRows(self, tree):
-        """ Return a list of tracks with all the associated tags, based on selected rows """
-        if tree.isRowSelected((0,)): return [tree.getItem(child, ROW_TRACK) for child in tree.iterChildren((0,))]
-        else:                        return [row[ROW_TRACK] for row in tree.getSelectedRows()]
+    def getTracksFromPaths(self, tree, paths):
+        """
+            Return a list of tracks with all the associated tags:
+                * From the list 'paths' if it is not None
+                * From the current selection if 'paths' is None
+        """
+        if paths is None:
+            if tree.isRowSelected((0,)): return [tree.getItem(child, ROW_TRACK) for child in tree.iterChildren((0,))]
+            else:                        return [row[ROW_TRACK] for row in tree.getSelectedRows()]
+        else:
+            if (0,) in paths: return [tree.getItem(child, ROW_TRACK) for child in tree.iterChildren((0,))]
+            else:             return [row[ROW_TRACK] for row in tree.getRows(paths)]
 
 
-    def playSelection(self, tree, replace):
-        """ Replace/extend the tracklist """
-        tracks = self.getTracksFromSelectedRows(tree)
+    def playPaths(self, tree, paths, replace):
+        """
+            Replace/extend the tracklist
+            If the list 'paths' is None, use the current selection
+        """
+        tracks = self.getTracksFromPaths(tree, paths)
 
         if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
         else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
@@ -164,7 +175,7 @@ class AudioCD(modules.ThreadedModule):
         self.tree.setItem((0,), ROW_NAME, rowName)
 
         # Optional information
-        try:    date = cddb['DYEAR'].strip().decode('iso-8859-15', 'replace')
+        try:    date = int(cddb['DYEAR'].strip().decode('iso-8859-15', 'replace'))
         except: date = None
 
         try:    genre = cddb['DGENRE'].strip().decode('iso-8859-15', 'replace')
@@ -262,7 +273,7 @@ class AudioCD(modules.ThreadedModule):
 
     def onDragDataGet(self, tree, context, selection, info, time):
         """ Provide information about the data being dragged """
-        serializedTracks = '\n'.join([track.serialize() for track in self.getTracksFromSelectedRows(tree)])
+        serializedTracks = '\n'.join([track.serialize() for track in self.getTracksFromPaths(tree, None)])
         selection.set(consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8, serializedTracks)
 
 
@@ -271,8 +282,8 @@ class AudioCD(modules.ThreadedModule):
         if self.popup is None:
             self.popup = tools.loadGladeFile('AudioCDMenu.glade')
             self.popup.get_widget('menu-popup').show_all()
-            self.popup.get_widget('item-add').connect('activate',     lambda widget: self.playSelection(tree, False))
-            self.popup.get_widget('item-play').connect('activate',    lambda widget: self.playSelection(tree, True))
+            self.popup.get_widget('item-add').connect('activate',     lambda widget: self.playPaths(tree, None, False))
+            self.popup.get_widget('item-play').connect('activate',    lambda widget: self.playPaths(tree, None, True))
             self.popup.get_widget('item-refresh').connect('activate', lambda widget: self.reloadDisc())
 
         self.popup.get_widget('menu-popup').popup(None, None, None, button, time)
@@ -282,8 +293,11 @@ class AudioCD(modules.ThreadedModule):
         """ A mouse button has been pressed """
         if event.button == 3:
             self.onShowPopupMenu(tree, event.button, event.time, path)
-        elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS and path is not None:
-            self.playSelection(tree, True)
+        elif path is not None:
+            if event.button == 2:
+                self.playPaths(tree, [path], False)
+            elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+                self.playPaths(tree, None, True)
 
 
     def onKeyPressed(self, tree, event):
@@ -296,7 +310,7 @@ class AudioCD(modules.ThreadedModule):
         elif keyname == 'Right':  tree.expandRows()
         elif keyname == 'minus':  tree.collapseRows()
         elif keyname == 'space':  tree.switchRows()
-        elif keyname == 'Return': self.playSelection(tree, True)
+        elif keyname == 'Return': self.playPaths(tree, None, True)
 
 
     # --== Configuration ==--
