@@ -28,7 +28,7 @@ from tools.log import logger
 MOD_INFO = ('AudioScrobbler', 'AudioScrobbler', _('Keep your Last.fm profile up to date'), [], False, False)
 
 CLI_ID         = 'dbl'
-CLI_VER        = '0.2'
+CLI_VER        = '0.3'
 MOD_NAME       = MOD_INFO[modules.MODINFO_NAME]
 PROTO_VER      = '1.2'
 AS_SERVER      = 'post.audioscrobbler.com'
@@ -101,7 +101,7 @@ class AudioScrobbler(modules.ThreadedModule):
         if not (track.hasArtist() and track.hasTitle() and track.hasLength()):
             return
 
-        if not(track.getLength() >= 30 and (self.currTrack[TRK_PLAY_TIME] >= 240 or self.currTrack[TRK_PLAY_TIME] >= track.getLength()/2)):
+        if not (track.getLength() >= 30 and (self.currTrack[TRK_PLAY_TIME] >= 240 or self.currTrack[TRK_PLAY_TIME] >= track.getLength()/2)):
             return
 
         params = (
@@ -120,16 +120,16 @@ class AudioScrobbler(modules.ThreadedModule):
 
 
     def getFromCache(self, howMany):
-        """ Return the last howMany tracks from the cache, replace the star with the correct index """
+        """ Return the oldest howMany tracks from the cache, replace the star with the correct index """
         if howMany > len(self.cache):
             howMany = len(self.cache)
 
-        return [self.cache[-howMany+i].replace('[*]', '[%d]' % i) for i in xrange(howMany)]
+        return [self.cache[i].replace('[*]', '[%d]' % i) for i in xrange(howMany)]
 
 
     def removeFromCache(self, howMany):
-        """ Remove the last howMany tracks from the cache """
-        self.cache[:] = self.cache[:-howMany]
+        """ Remove the oldest howMany tracks from the cache """
+        self.cache[:] = self.cache[howMany:]
 
 
     def getCacheSize(self):
@@ -148,11 +148,11 @@ class AudioScrobbler(modules.ThreadedModule):
 
     def handshake(self):
         """ Authenticate the user to the submission servers, return True if OK """
-        timestamp       = int(time())
+        now             = int(time())
         self.session[:] = [None, None, None]
 
         # Postpone or cancel this handshake?
-        if self.isBanned or timestamp - self.lastHandshake < self.handshakeDelay:
+        if self.isBanned or (now - self.lastHandshake) < self.handshakeDelay:
             return False
 
         # Asking for login information must be done in the GTK main loop, because a dialog box might be displayed if needed
@@ -160,10 +160,10 @@ class AudioScrobbler(modules.ThreadedModule):
         if self.passwd is None:
             return False
 
-        token              = md5.md5('%s%d' % (md5.md5(self.passwd).hexdigest(), timestamp)).hexdigest()
+        token              = md5.md5('%s%d' % (md5.md5(self.passwd).hexdigest(), now)).hexdigest()
         self.passwd        = None
-        request            = 'http://%s/?hs=true&p=%s&c=%s&v=%s&u=%s&t=%d&a=%s' % (AS_SERVER, PROTO_VER, CLI_ID, CLI_VER, self.login, timestamp, token)
-        self.lastHandshake = timestamp
+        request            = 'http://%s/?hs=true&p=%s&c=%s&v=%s&u=%s&t=%d&a=%s' % (AS_SERVER, PROTO_VER, CLI_ID, CLI_VER, self.login, now, token)
+        self.lastHandshake = now
 
         try:
             hardFailure = False
@@ -238,12 +238,13 @@ class AudioScrobbler(modules.ThreadedModule):
             return False
 
         try:
-            hardFailure = False
-            data        = 's=%s&%s' % (self.session[SESSION_ID], '&'.join(self.getFromCache(MAX_SUBMISSION)))
-            reply       = urlopen(self.session[SUBMISSION_URL], data).read().strip().split('\n')
+            hardFailure  = False
+            cachedTracks = self.getFromCache(MAX_SUBMISSION)
+            data         = 's=%s&%s' % (self.session[SESSION_ID], '&'.join(cachedTracks))
+            reply        = urlopen(self.session[SUBMISSION_URL], data).read().strip().split('\n')
 
             if reply[0] == 'OK':
-                self.removeFromCache(MAX_SUBMISSION)
+                self.removeFromCache(len(cachedTracks))
                 return True
 
             elif reply[0] == 'BADSESSION' and firstTry:
