@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import gtk.gdk, gui, modules, os, socket, tempfile, tools, traceback, urllib, urllib2
+import gtk.gdk, gui, modules, os, socket, tempfile, tools, traceback, urllib2
 
 from tools     import consts, prefs
 from gettext   import gettext as _
@@ -32,13 +32,24 @@ AS_API_KEY   = 'fd8dd98d26bb3f288f3e626502f9add6'   # Ingelrest François' Audio
 AS_TAG_START = '<image size="large">'               # The text that is right before the URL to the cover
 AS_TAG_END   = '</image>'                           # The text that is right after the URL to the cover
 
-# It seems that a non standard 'user-agent' header may cause problem, so let's cheat a bit
+# It seems that a non standard 'user-agent' header may cause problem, so let's cheat
 USER_AGENT = 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/2008072820 Firefox/3.0.1'
 
-COVER_WIDTH   = 74   # Width allocated to covers in the model
-COVER_HEIGHT  = 74   # Height allocated to covers in the model
-COVER_OFFSETX =  8   # X-offset to render the cover in the model
-COVER_OFFSETY =  0   # Y-offset to render the cover in the model
+# We store both the paths to the thumbnail and to the full size image
+(
+    CVR_THUMB,
+    CVR_FULL
+) = range(2)
+
+# Constants for thumbnails
+THUMBNAIL_WIDTH   = 74   # Width allocated to thumbnails in the model
+THUMBNAIL_HEIGHT  = 74   # Height allocated to thumbnails in the model
+THUMBNAIL_OFFSETX =  8   # X-offset to render the thumbnail in the model
+THUMBNAIL_OFFSETY =  0   # Y-offset to render the thumbnail in the model
+
+# Constants for full size covers
+FULL_SIZE_COVER_WIDTH  = 300
+FULL_SIZE_COVER_HEIGHT = 300
 
 # File formats we can read
 ACCEPTED_FILE_FORMATS = {'.jpg': None, '.jpeg': None, '.png': None, '.gif': None}
@@ -49,7 +60,7 @@ PREFS_DFT_PREFER_USER_COVERS   = True
 PREFS_DFT_USER_COVER_FILENAMES = ['album', 'art', 'cover', 'front']
 
 # Since it's very small, we integrate the PNG file directly in the source code
-COVER_MODEL = '\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00R\x00\x00\x00J\x08\x00\x00\x00\x00\r\xc2\\\xb1\x00\x00\x00\x02bKGD\x00\xff\x87\x8f\xcc\xbf\x00\x00\x00vIDATX\xc3\xed\x97\xd1\r\x80@\x08C\x1bFq\x08\xf7qOg\x13s\x9e\x1b\xc8%\x92\xbe\xf2\xdf\x10z\xb4G\x1c\x13{\x96!\xce\x07*Dl\x1aUJ\xa9r4\xa1\x9c*)\x7f\xde\xa5J[\xb4V\xdc\x96\xf2\xcaQ\x99\xcc\xd2\xc6\xdcZ\xd8\xc6\x9b\x94(\x0e\xe5\xe7GD\x8e\x9b\xc5\x99\x883\x9b\xa7\xdec\xc7\xdds\x1cy8\x9e\xf1K\x0e>v\x9c/+\xe6\xc6,\xd7\x9a[!\xe5\r(\xf6\xc6u\xfe\xcbvu\x00\x00\x00\x00IEND\xaeB`\x82'
+THUMBNAIL_MODEL = '\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00R\x00\x00\x00J\x08\x00\x00\x00\x00\r\xc2\\\xb1\x00\x00\x00\x02bKGD\x00\xff\x87\x8f\xcc\xbf\x00\x00\x00vIDATX\xc3\xed\x97\xd1\r\x80@\x08C\x1bFq\x08\xf7qOg\x13s\x9e\x1b\xc8%\x92\xbe\xf2\xdf\x10z\xb4G\x1c\x13{\x96!\xce\x07*Dl\x1aUJ\xa9r4\xa1\x9c*)\x7f\xde\xa5J[\xb4V\xdc\x96\xf2\xcaQ\x99\xcc\xd2\xc6\xdcZ\xd8\xc6\x9b\x94(\x0e\xe5\xe7GD\x8e\x9b\xc5\x99\x883\x9b\xa7\xdec\xc7\xdds\x1cy8\x9e\xf1K\x0e>v\x9c/+\xe6\xc6,\xd7\x9a[!\xe5\r(\xf6\xc6u\xfe\xcbvu\x00\x00\x00\x00IEND\xaeB`\x82'
 
 
 class Covers(modules.ThreadedModule):
@@ -62,12 +73,12 @@ class Covers(modules.ThreadedModule):
 
     def onModLoaded(self):
         """ The module has been loaded """
-        self.cfgWin         = None                                   # Configuration window
-        self.coverMap       = {}                                     # Map albums to covers
-        self.currTrack      = None                                   # The current track being played, if any
-        self.modelPath      = None                                   # Path to the model we use to generate the covers
-        self.cacheRootPath  = os.path.join(consts.dirCfg, MOD_NAME)  # Cache for downloaded covers
-        self.coverBlacklist = {}                                     # When a cover cannot be downloaded, avoid requesting it again
+        self.cfgWin             = None                                   # Configuration window
+        self.coverMap           = {}                                     # Store covers previously requested
+        self.currTrack          = None                                   # The current track being played, if any
+        self.cacheRootPath      = os.path.join(consts.dirCfg, MOD_NAME)  # Local cache for Internet covers
+        self.coverBlacklist     = {}                                     # When a cover cannot be downloaded, avoid requesting it again
+        self.thumbnailModelPath = None                                   # Path to the model we use to generate the covers
 
         if not os.path.exists(self.cacheRootPath):
             os.mkdir(self.cacheRootPath)
@@ -76,63 +87,97 @@ class Covers(modules.ThreadedModule):
     def onModUnloaded(self):
         """ The module has been unloaded """
         if self.currTrack is not None:
-            modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': self.currTrack, 'path': None})
+            modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': self.currTrack, 'pathThumbnail': None, 'pathFullSize': None})
 
-        # Delete covers
-        for cover in self.coverMap.itervalues():
-            if os.path.exists(cover):
-                os.remove(cover)
+        # Delete covers that have been generated by this module
+        for covers in self.coverMap.itervalues():
+            if os.path.exists(covers[CVR_THUMB]):
+                os.remove(covers[CVR_THUMB])
+            if os.path.exists(covers[CVR_FULL]):
+                os.remove(covers[CVR_FULL])
         self.coverMap = None
 
-        # Delete the cover model
-        if self.modelPath is not None and os.path.exists(self.modelPath):
-            os.remove(self.modelPath)
-        self.modelPath = None
+        # Delete the cover model if any
+        if self.thumbnailModelPath is not None and os.path.exists(self.thumbnailModelPath):
+            os.remove(self.thumbnailModelPath)
+        self.thumbnailModelPath = None
 
         # Delete blacklist
         self.coverBlacklist = None
 
 
-    def prepareCover(self, inFile, outFile):
-        """ Finalize inFile (e.g., resize it) and write it to outFile (outFile and inFile may be equal) """
+    def generateFullSizeCover(self, inFile, outFile):
+        """ Resize inFile if needed, and write it to outFile (outFile and inFile may be equal) """
         try:
             cover = gtk.gdk.pixbuf_new_from_file(inFile)
 
             # Resize values
-            if cover.get_width() <= COVER_WIDTH:
-                destX     = (COVER_WIDTH - cover.get_width()) / 2 + COVER_OFFSETX
+            if cover.get_width() <= FULL_SIZE_COVER_WIDTH:
                 scaleX    = 1.0
-                offsetX   = (COVER_WIDTH - cover.get_width()) / 2 + COVER_OFFSETX
-                destWidth = COVER_WIDTH
+                destWidth = cover.get_width()
             else:
-                destX     = COVER_OFFSETX
-                scaleX    = float(COVER_WIDTH) / float(cover.get_width())
-                offsetX   = COVER_OFFSETX
-                destWidth = COVER_WIDTH
+                scaleX    = float(FULL_SIZE_COVER_WIDTH) / float(cover.get_width())
+                destWidth = FULL_SIZE_COVER_WIDTH
 
-            if cover.get_height() <= COVER_HEIGHT:
-                destY      = (COVER_HEIGHT - cover.get_height()) / 2 + COVER_OFFSETY
+            if cover.get_height() <= FULL_SIZE_COVER_HEIGHT:
                 scaleY     = 1.0
-                offsetY    = (COVER_HEIGHT - cover.get_height()) / 2 + COVER_OFFSETY
-                destHeight = COVER_HEIGHT
+                destHeight = cover.get_height()
             else:
-                destY      = COVER_OFFSETY
-                scaleY     = float(COVER_HEIGHT) / float(cover.get_height())
-                offsetY    = COVER_OFFSETY
-                destHeight = COVER_HEIGHT
+                scaleY     = float(FULL_SIZE_COVER_HEIGHT) / float(cover.get_height())
+                destHeight = FULL_SIZE_COVER_HEIGHT
+
+            if scaleX != 1 or scaleY != 1:
+                # Alpha channel is needed to prevent errors when the source cover has one
+                fullSizeCover = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, destWidth, destHeight)
+                cover.scale(fullSizeCover, 0, 0, destWidth, destHeight, 0, 0, scaleX, scaleY, gtk.gdk.INTERP_HYPER)
+                fullSizeCover.save(outFile, 'jpeg', {'quality':'100'})
+            else:
+                cover.save(outFile, 'jpeg', {'quality':'100'})
+        except:
+            logger.error('[%s] An error occurred while generating a showable full size cover\n\n%s' % (MOD_NAME, traceback.format_exc()))
+
+
+    def generateThumbnail(self, inFile, outFile):
+        """ Generate a thumbnail from inFile (e.g., resize it) and write it to outFile (outFile and inFile may be equal) """
+        try:
+            cover = gtk.gdk.pixbuf_new_from_file(inFile)
+
+            # Resize values
+            if cover.get_width() <= THUMBNAIL_WIDTH:
+                destX     = (THUMBNAIL_WIDTH - cover.get_width()) / 2 + THUMBNAIL_OFFSETX
+                scaleX    = 1.0
+                offsetX   = (THUMBNAIL_WIDTH - cover.get_width()) / 2 + THUMBNAIL_OFFSETX
+                destWidth = THUMBNAIL_WIDTH
+            else:
+                destX     = THUMBNAIL_OFFSETX
+                scaleX    = float(THUMBNAIL_WIDTH) / float(cover.get_width())
+                offsetX   = THUMBNAIL_OFFSETX
+                destWidth = THUMBNAIL_WIDTH
+
+            if cover.get_height() <= THUMBNAIL_HEIGHT:
+                destY      = (THUMBNAIL_HEIGHT - cover.get_height()) / 2 + THUMBNAIL_OFFSETY
+                scaleY     = 1.0
+                offsetY    = (THUMBNAIL_HEIGHT - cover.get_height()) / 2 + THUMBNAIL_OFFSETY
+                destHeight = THUMBNAIL_HEIGHT
+            else:
+                destY      = THUMBNAIL_OFFSETY
+                scaleY     = float(THUMBNAIL_HEIGHT) / float(cover.get_height())
+                offsetY    = THUMBNAIL_OFFSETY
+                destHeight = THUMBNAIL_HEIGHT
 
             # Generate the model we use, if needed
-            if self.modelPath is None or not os.path.exists(self.modelPath):
-                self.modelPath = tempfile.mktemp() + '.png'
-                output = open(self.modelPath, 'wb')
-                output.write(COVER_MODEL)
+            if self.thumbnailModelPath is None or not os.path.exists(self.thumbnailModelPath):
+                self.thumbnailModelPath = tempfile.mktemp() + '.png'
+                output = open(self.thumbnailModelPath, 'wb')
+                output.write(THUMBNAIL_MODEL)
                 output.close()
 
-            coverModel = gtk.gdk.pixbuf_new_from_file(self.modelPath)
+            # Alpha channel is needed to prevent errors when the source cover has one
+            coverModel = gtk.gdk.pixbuf_new_from_file(self.thumbnailModelPath).add_alpha(False, chr(255), chr(255), chr(255))
             cover.scale(coverModel, destX, destY, destWidth, destHeight, offsetX, offsetY, scaleX, scaleY, gtk.gdk.INTERP_HYPER)
             coverModel.save(outFile, 'jpeg', {'quality':'100'})
         except:
-            logger.error('[%s] An error occurred while preparing the cover\n\n%s' % (MOD_NAME, traceback.format_exc()))
+            logger.error('[%s] An error occurred while generating a thumbnail\n\n%s' % (MOD_NAME, traceback.format_exc()))
 
 
     def getUserCover(self, trackPath):
@@ -270,7 +315,7 @@ class Covers(modules.ThreadedModule):
         """ A new track is being played, try to retrieve the corresponding cover """
         # Make sure we have enough information
         if track.getArtist() == consts.UNKNOWN_ARTIST or track.getAlbum() == consts.UNKNOWN_ALBUM:
-            modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'path': None})
+            modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'pathThumbnail': None, 'pathFullSize': None})
             return
 
         album          = track.getAlbum().lower()
@@ -280,12 +325,15 @@ class Covers(modules.ThreadedModule):
         self.currTrack = track
 
         # Let's see whether we already have the cover
-        try:
-            if os.path.exists(self.coverMap[coverKey]):
-                modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'path': self.coverMap[coverKey]})
+        if coverKey in self.coverMap:
+            covers        = self.coverMap[coverKey]
+            pathFullSize  = covers[CVR_FULL]
+            pathThumbnail = covers[CVR_THUMB]
+
+            # Make sure the files are still there
+            if os.path.exists(pathThumbnail) and os.path.exists(pathFullSize):
+                modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'pathThumbnail': pathThumbnail, 'pathFullSize': pathFullSize})
                 return
-        except:
-            pass
 
         # Should we check for a user cover?
         if not prefs.get(__name__, 'download-covers', PREFS_DFT_DOWNLOAD_COVERS)        \
@@ -298,21 +346,23 @@ class Covers(modules.ThreadedModule):
 
         # If we still don't have a cover, maybe we can try to download it
         if rawCover is None:
-            modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'path': None})
+            modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'pathThumbnail': None, 'pathFullSize': None})
 
             if prefs.get(__name__, 'download-covers', PREFS_DFT_DOWNLOAD_COVERS):
                 rawCover = self.getFromInternet(artist, album)
 
         # If we still don't have a cover, too bad
-        # Otherwise, prepare it and add it to our cover map
+        # Otherwise, generate a thumbnail and a full size cover, and add it to our cover map
         if rawCover is not None:
-            cover = tempfile.mktemp() + '.jpg'
-            self.prepareCover(rawCover, cover)
-            if os.path.exists(cover):
-                self.coverMap[coverKey] = cover
-                modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'path': cover})
+            thumbnail     = tempfile.mktemp() + '.jpg'
+            fullSizeCover = tempfile.mktemp() + '.jpg'
+            self.generateThumbnail(rawCover, thumbnail)
+            self.generateFullSizeCover(rawCover, fullSizeCover)
+            if os.path.exists(thumbnail) and os.path.exists(fullSizeCover):
+                self.coverMap[coverKey] = (thumbnail, fullSizeCover)
+                modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'pathThumbnail': thumbnail, 'pathFullSize': fullSizeCover})
             else:
-                modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'path': None})
+                modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'pathThumbnail': None, 'pathFullSize': None})
 
 
     # --== Message handler ==--
