@@ -18,7 +18,7 @@
 
 import gtk, modules, tools
 
-from tools   import consts, prefs
+from tools   import consts, prefs, sec2str
 from gettext import gettext as _
 
 MOD_INFO = ('Control Panel', 'Control Panel', '', [], True, False)
@@ -102,10 +102,10 @@ class CtrlPanel(modules.Module):
     def onNewTrackPosition(self, elapsedTime):
         """ The track position has changed """
         if not self.sclBeingDragged:
-            self.lblElapsed.set_label(tools.sec2str(elapsedTime))
+            self.lblElapsed.set_label(sec2str(elapsedTime))
             if elapsedTime >= self.currTrackLength:
                 elapsedTime = self.currTrackLength
-            self.lblRemaining.set_label(tools.sec2str(self.currTrackLength - elapsedTime))
+            self.lblRemaining.set_label(sec2str(self.currTrackLength - elapsedTime))
             # Make sure the handler will not be called
             self.sclSeek.disconnect(self.seekHandler)
             self.sclSeek.set_value(elapsedTime)
@@ -153,8 +153,8 @@ class CtrlPanel(modules.Module):
         if value >= self.currTrackLength: value = self.currTrackLength
         else:                             value = int(value)
 
-        self.lblElapsed.set_label(tools.sec2str(value))
-        self.lblRemaining.set_label(tools.sec2str(self.currTrackLength - value))
+        self.lblElapsed.set_label(sec2str(value))
+        self.lblRemaining.set_label(sec2str(self.currTrackLength - value))
 
 
     def onVolumeValueChanged(self, button, value):
@@ -167,7 +167,8 @@ class CtrlPanel(modules.Module):
 
     def handleMsg(self, msg, params):
         """ Handle message sent to this module """
-        if   msg == consts.MSG_EVT_PAUSED:         self.onPaused()
+        if   msg == consts.MSG_EVT_TRACK_POSITION: self.onNewTrackPosition(params['seconds'])
+        elif msg == consts.MSG_EVT_PAUSED:         self.onPaused()
         elif msg == consts.MSG_EVT_STOPPED:        self.onStopped()
         elif msg == consts.MSG_EVT_UNPAUSED:       self.onUnpaused()
         elif msg == consts.MSG_EVT_APP_QUIT:       prefs.set(__name__, 'volume', self.btnVolume.get_value())
@@ -175,5 +176,4 @@ class CtrlPanel(modules.Module):
         elif msg == consts.MSG_EVT_APP_STARTED:    self.onAppStarted()
         elif msg == consts.MSG_EVT_TRACK_MOVED:    self.onTrackMoved(params['hasPrevious'], params['hasNext'])
         elif msg == consts.MSG_EVT_NEW_TRACKLIST:  self.btnPlay.set_sensitive(len(params['tracks']) != 0)
-        elif msg == consts.MSG_EVT_TRACK_POSITION: self.onNewTrackPosition(params['seconds'])
         elif msg == consts.MSG_EVT_VOLUME_CHANGED: self.onVolumeChanged(params['value'])
