@@ -61,35 +61,41 @@ class FileExplorer(modules.Module):
 
     def onModLoaded(self):
         """ The module has been loaded """
-        columns = (('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf), (gtk.CellRendererText(), TYPE_STRING)], True),
-                   (None, [(None, TYPE_INT)],                                                                  False),
-                   (None, [(None, TYPE_STRING)],                                                               False))
-
-        self.tree            = extTreeview.ExtTreeView(columns, True)
+        self.trees           = {}     # A [tree, state] per root folder
         self.popup           = None
         self.cfgWin          = None
-        self.folders         = prefs.get(__name__, 'media-folders',     PREFS_DEFAULT_MEDIA_FOLDERS)
+        self.folders         = prefs.get(__name__, 'media-folders', PREFS_DEFAULT_MEDIA_FOLDERS)
         self.scrolled        = gtk.ScrolledWindow()
         self.currFolder      = None
         self.showHiddenFiles = prefs.get(__name__, 'show-hidden-files', PREFS_DEFAULT_SHOW_HIDDEN_FILES)
         # Explorer
-        self.tree.setIsDraggableFunc(self.isPlayable)
-        self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_URI]])
-        self.scrolled.add(self.tree)
         self.scrolled.set_shadow_type(gtk.SHADOW_IN)
         self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         self.scrolled.show()
-        # GTK handlers
-        self.tree.connect('drag-data-get',              self.onDragDataGet)
-        self.tree.connect('key-press-event',            self.onKeyPressed)
-        self.tree.connect('exttreeview-row-expanded',   self.onRowExpanded)
-        self.tree.connect('exttreeview-row-collapsed',  self.onRowCollapsed)
-        self.tree.connect('exttreeview-button-pressed', self.onMouseButton)
 
         for name in self.folders:
             modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': name, 'icon': None, 'widget': self.scrolled})
 
 
+    def createTree(self, rootFolder):
+        """ Return a new tree for the given root folder """
+        columns = (('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf), (gtk.CellRendererText(), TYPE_STRING)], True),
+                   (None, [(None, TYPE_INT)],                                                                  False),
+                   (None, [(None, TYPE_STRING)],                                                               False))
+
+        tree = extTreeview.ExtTreeView(columns, True)
+        tree.setIsDraggableFunc(self.isPlayable)
+        tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_URI]])
+        # GTK handlers
+        tree.connect('drag-data-get',              self.onDragDataGet)
+        tree.connect('key-press-event',            self.onKeyPressed)
+        tree.connect('exttreeview-row-expanded',   self.onRowExpanded)
+        tree.connect('exttreeview-row-collapsed',  self.onRowCollapsed)
+        tree.connect('exttreeview-button-pressed', self.onMouseButton)
+
+        return tree
+
+
     def onModUnloaded(self):
         """ The module is going to be unloaded """
         prefs.set(__name__, 'media-folders',     self.folders)
@@ -103,7 +109,7 @@ class FileExplorer(modules.Module):
 
     def isPlayable(self):
         """ Only playable rows can be dragged """
-        for row in self.tree.iterSelectedRows():
+        for row in self.trees[self.currFolder][0].iterSelectedRows():
             if row[ROW_TYPE] in (TYPE_FILE, TYPE_PLIST) or (row[ROW_TYPE] == TYPE_DIR and row[ROW_PIXBUF] == consts.icoMediaDir):
                 return True
         return False
@@ -117,27 +123,47 @@ class FileExplorer(modules.Module):
                 self.cfgWin.getWidget('chk-hidden').set_active(showHiddenFiles)
 
             self.showHiddenFiles = showHiddenFiles
-            self.refresh(self.tree)
+            self.refresh(self.trees[self.currFolder][0])
 
 
-    def getFilesFromSelection(self, tree):
-        """ Return a list of playable files from the selected rows """
-        return [row[ROW_FULLPATH] for row in tree.getSelectedRows() if row[ROW_TYPE] != TYPE_NONE]
+    def getFilesFromPaths(self, tree, paths):
+        """
+            Return a list of playable files from:
+                * The list 'paths' if it is not None
+                * The selected rows if paths is None
+        """
+        if paths is None: return [row[ROW_FULLPATH] for row in tree.getSelectedRows() if row[ROW_TYPE] != TYPE_NONE]
+        else:             return [row[ROW_FULLPATH] for row in tree.getRows(paths) if row[ROW_TYPE] != TYPE_NONE]
 
 
-    def playSelection(self, tree, replace):
-        """ Replace/extend the tracklist """
+    def playPaths(self, tree, paths, replace):
+        """
+            Replace/extend the tracklist
+            If the list 'paths' is None, use the current selection
+        """
         if self.isPlayable():
-            tracks = media.getTracks(self.getFilesFromSelection(tree))
+            tracks = media.getTracks(self.getFilesFromPaths(tree, paths))
             if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
             else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
 
 
     def renameFolder(self, oldName, newName):
         """ Rename a folder """
+        # Remove immediately the tree from the scrolled window if needed
+        # We can't do that later, because we have no longer access to the old name after that
+        if self.currFolder == oldName:
+            self.scrolled.remove(self.trees[oldName][0])
+            self.currFolder = None
+
+        # Rename the root folder
         self.folders[newName] = self.folders[oldName]
         del self.folders[oldName]
 
+        # Move the tree associated to that folder, if any
+        if oldName in self.trees:
+            self.trees[newName] = self.trees[oldName]
+            del self.trees[oldName]
+
         modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': oldName})
         modules.postMsg(consts.MSG_CMD_EXPLORER_ADD,    {'modName': MOD_L10N, 'expName': newName, 'icon': None, 'widget': self.scrolled})
 
@@ -148,7 +174,7 @@ class FileExplorer(modules.Module):
     def __startLoading(self, tree, row):
         """ Tell the user that the contents of row is being loaded """
         name = tree.getItem(row, ROW_NAME)
-        # TODO Put this style somewhere, we're using it at multiple places
+        # TODO Put this style somewhere, we're using it in multiple places
         tree.setItem(row, ROW_NAME, '%s  <span size="smaller" foreground="#909090">%s</span>' % (name, _('loading...')))
 
 
@@ -288,10 +314,13 @@ class FileExplorer(modules.Module):
         """ A mouse button has been pressed """
         if event.button == 3:
             self.onShowPopupMenu(tree, event.button, event.time, path)
-        elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS and path is not None:
-            if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.playSelection(tree, True)
-            elif tree.row_expanded(path):                         tree.collapse_row(path)
-            else:                                                 tree.expand_row(path, False)
+        elif path is not None:
+            if event.button == 2:
+                 self.playPaths(tree, [path], False)
+            elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+                if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.playPaths(tree, None, True)
+                elif tree.row_expanded(path):                         tree.collapse_row(path)
+                else:                                                 tree.expand_row(path, False)
 
 
     def onShowPopupMenu(self, tree, button, time, path):
@@ -300,8 +329,8 @@ class FileExplorer(modules.Module):
             self.popup = tools.loadGladeFile('FileExplorerMenu.glade')
             self.popup.get_widget('item-refresh').connect('activate', lambda widget: self.refresh(tree))
             self.popup.get_widget('item-collapse').connect('activate', lambda widget: tree.collapse_all())
-            self.popup.get_widget('item-add').connect('activate', lambda widget: self.playSelection(tree, False))
-            self.popup.get_widget('item-play').connect('activate', lambda widget: self.playSelection(tree, True))
+            self.popup.get_widget('item-add').connect('activate', lambda widget: self.playPaths(tree, None, False))
+            self.popup.get_widget('item-play').connect('activate', lambda widget: self.playPaths(tree, None, True))
             self.popup.get_widget('item-hidden').connect('toggled', lambda item: self.setShowHiddenFiles(item.get_active()))
             self.popup.get_widget('menu-popup').show_all()
 
@@ -323,7 +352,7 @@ class FileExplorer(modules.Module):
         elif keyname == 'Right':  tree.expandRows()
         elif keyname == 'minus':  tree.collapseRows()
         elif keyname == 'space':  tree.switchRows()
-        elif keyname == 'Return': self.playSelection(tree, True)
+        elif keyname == 'Return': self.playPaths(tree, None, True)
 
 
     def onRowExpanded(self, tree, path):
@@ -340,7 +369,7 @@ class FileExplorer(modules.Module):
 
     def onDragDataGet(self, tree, context, selection, info, time):
         """ Provide information about the data being dragged """
-        selection.set('text/uri-list', 8, ' '.join([urllib.pathname2url(file) for file in self.getFilesFromSelection(tree)]))
+        selection.set('text/uri-list', 8, ' '.join([urllib.pathname2url(file) for file in self.getFilesFromPaths(tree, None)]))
 
 
    # --== Message handler ==--
@@ -349,11 +378,32 @@ class FileExplorer(modules.Module):
     def handleMsg(self, msg, params):
         """ Handle messages sent to this module """
         if msg == consts.MSG_EVT_EXPLORER_CHANGED and params['modName'] == MOD_L10N and self.currFolder != params['expName']:
+            newFolder = params['expName']
+
+            # Create a tree for this root folder if there's not already one
+            if newFolder not in self.trees:
+                self.trees[newFolder] = [self.createTree(newFolder), None]
+
+            tree, state = self.trees[newFolder]
+
+            # Make sure this tree is the current child of the scrolled window
+            if self.currFolder is not None:
+                currTree = self.trees[self.currFolder][0]
+                self.trees[self.currFolder] = [currTree, currTree.saveState(ROW_NAME)]
+                self.scrolled.remove(currTree)
+            self.scrolled.add(tree)
+
+            # Display the contents of the root folder
             self.currFolder = params['expName']
-            self.tree.clear()
-            self.exploreDir(self.tree, None, self.folders[self.currFolder])
-            if len(self.tree) != 0:
-                self.tree.scroll_to_cell(0)
+
+            # If the tree has just been created, we need to explore the root folder to populate the tree
+            if state is not None:
+                # FIXME This doesn't work, the scrollbar stays at the top...
+                idle_add(tree.restoreState, state, ROW_NAME)
+            else:
+                self.exploreDir(tree, None, self.folders[self.currFolder])
+                if len(tree) != 0:
+                    tree.scroll_to_cell(0)
         elif msg == consts.MSG_EVT_APP_STARTED:
             self.onModLoaded()
         elif msg == consts.MSG_EVT_APP_QUIT:
@@ -370,10 +420,10 @@ class FileExplorer(modules.Module):
             # Create the list of folders
             txtRdr  = gtk.CellRendererText()
             pixRdr  = gtk.CellRendererPixbuf()
-            columns = ((None, [(txtRdr, TYPE_STRING)],                           0, False),
-                       ('',   [(pixRdr, gtk.gdk.Pixbuf), (txtRdr, TYPE_STRING)], 2, False))
+            columns = ((None, [(txtRdr, TYPE_STRING)],                           0, False, False),
+                       ('',   [(pixRdr, gtk.gdk.Pixbuf), (txtRdr, TYPE_STRING)], 2, False, True))
 
-            self.cfgList = extListview.ExtListView(columns, sortable=False, useMarkup=True)
+            self.cfgList = extListview.ExtListView(columns, sortable=False, useMarkup=True, canShowHideColumns=False)
             self.cfgList.set_headers_visible(False)
             self.cfgWin.getWidget('scrolledwindow1').add(self.cfgList)
             # Connect handlers
@@ -425,8 +475,16 @@ class FileExplorer(modules.Module):
                 name = row[0]
                 modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': name})
                 del self.folders[name]
+
+                # Remove the tree, if any, from the scrolled window
                 if self.currFolder == name:
+                    self.scrolled.remove(self.trees[name][0])
                     self.currFolder = None
+
+                # Remove the tree associated to this root folder, if any
+                if name in self.trees:
+                    del self.trees[name]
+
             self.cfgList.removeSelectedRows()
 
 
