@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ class GSTPlayer(modules.Module):
         """ This is the real initialization function, called when this module has been loaded """
         self.timer       = None    # Used to monitor the current track
         self.bufPos      = None    # Used when the user wants to seek a position while paused
-        self.bufFile     = None    # If not None, the file that the second player is going to play
+        self.bufURI      = None    # If not None, the resource that the second player is going to play
         self.eosSignaled = False   # Prevent duplicated EOS signals
         # We use two players to avoid gaps between songs
         self.player  = gst.element_factory_make('playbin', consts.appName + '1')
@@ -69,23 +69,23 @@ class GSTPlayer(modules.Module):
         modules.postMsg(consts.MSG_EVT_TRACK_POSITION, {'seconds': int(currPosition / 1000000000)})
 
         # If the current track is close to its end, we must ask to buffer the next one, if any
-        # If the track is *really* close to its end, start playing the buffered file, if any
+        # If the track is *really* close to its end, start playing the buffered resource, if any
         remaining = self.player.query_duration(gst.FORMAT_TIME)[0] - currPosition
-        if remaining <= 4000000000 and self.bufFile is None:
+        if remaining <= 4000000000 and self.bufURI is None:
             modules.postMsg(consts.MSG_EVT_NEED_BUFFER)
-        elif remaining <= 250000000 and self.bufFile is not None and self.player2.get_state()[1] != gst.STATE_PLAYING:
+        elif remaining <= 250000000 and self.bufURI is not None and self.player2.get_state()[1] != gst.STATE_PLAYING:
             self.player2.set_state(gst.STATE_PLAYING)
 
         return True
 
 
-    def bufferFile(self, file):
-        """ Buffer the given file with the second player """
-        if self.bufFile is None:
-            self.bufFile = file
-            self.player2.set_property('uri', file)
+    def bufferResource(self, uri):
+        """ Buffer the given resource with the second player """
+        if self.bufURI is None:
+            self.bufURI = uri
+            self.player2.set_property('uri', uri)
             self.player2.set_state(gst.STATE_READY)
-            # Increase the frequency of the timer, to start playing the buffered file just when needed
+            # Increase the frequency of the timer, to start playing the buffered resource just when needed
             if self.timer is not None:
                 gobject.source_remove(self.timer)
             self.timer = gobject.timeout_add(100, self.timerFunc)
@@ -93,20 +93,24 @@ class GSTPlayer(modules.Module):
 
     def onGSTMsg(self, bus, msg):
         """ GStreamer message handler """
-        if (msg.type == gst.MESSAGE_EOS or msg.type == gst.MESSAGE_ERROR) and not self.eosSignaled:
+        if msg.type in (gst.MESSAGE_EOS, gst.MESSAGE_ERROR) and not self.eosSignaled:
             self.eosSignaled = True
-            modules.postMsg(consts.MSG_EVT_TRACK_ENDED)
+
+            if msg.type == gst.MESSAGE_EOS: modules.postMsg(consts.MSG_EVT_TRACK_ENDED_OK)
+            else:                           modules.postMsg(consts.MSG_EVT_TRACK_ENDED_ERROR)
+
             return True
+
         return False
 
 
-    def play(self, file):
-        """ Play the given file """
+    def play(self, uri):
+        """ Play the given resource """
         self.bufPos      = None
         self.eosSignaled = False
         self.player.set_state(gst.STATE_NULL)
-        # If the given file is already buffered, we're lucky and we just need to be sure that the second player is playing it
-        if file == self.bufFile:
+        # If the given resource is already buffered, we're lucky and we just need to be sure that the second player is playing it
+        if uri == self.bufURI:
             if self.player2.get_state()[1] != gst.STATE_PLAYING:
                 self.player2.set_state(gst.STATE_PLAYING)
             # Now we need to swap the two players
@@ -115,14 +119,14 @@ class GSTPlayer(modules.Module):
             self.bus = self.player.get_bus()
             self.bus.add_signal_watch()
             self.bus.connect('message', self.onGSTMsg)
-        # No luck, the buffered file, if any, is not the correct one
+        # No luck, the buffered resource, if any, is not the correct one
         else:
             if self.player2.get_state()[1] != gst.STATE_NULL:
                 self.player2.set_state(gst.STATE_NULL)
-            self.player.set_property('uri', file)
+            self.player.set_property('uri', uri)
             self.player.set_state(gst.STATE_PLAYING)
         # Clear the buffer in any case
-        self.bufFile = None
+        self.bufURI = None
         # Ensure that the timer frequency is back to normal
         if self.timer is not None:
             gobject.source_remove(self.timer)
@@ -137,7 +141,7 @@ class GSTPlayer(modules.Module):
 
         self.player.set_state(gst.STATE_NULL)
         self.player2.set_state(gst.STATE_NULL)
-        self.bufFile = None
+        self.bufURI = None
         modules.postMsg(consts.MSG_EVT_STOPPED)
 
 
@@ -173,9 +177,9 @@ class GSTPlayer(modules.Module):
     def handleMsg(self, msg, params):
         """ Handle messages sent to this module """
         if   msg == consts.MSG_CMD_STOP:         self.stop()
-        elif msg == consts.MSG_CMD_PLAY:         self.play(params['filename'])
+        elif msg == consts.MSG_CMD_PLAY:         self.play(params['uri'])
         elif msg == consts.MSG_CMD_SEEK:         self.seek(params['seconds'])
-        elif msg == consts.MSG_CMD_BUFFER:       self.bufferFile(params['filename'])
+        elif msg == consts.MSG_CMD_BUFFER:       self.bufferResource(params['uri'])
         elif msg == consts.MSG_CMD_SET_VOLUME:   self.setVolume(params['value'])
         elif msg == consts.MSG_EVT_APP_STARTED:  self.onAppStarted()
         elif msg == consts.MSG_CMD_TOGGLE_PAUSE: self.togglePause()
