@@ -97,12 +97,29 @@ class Library(modules.Module):
 
     def onAppStarted(self):
         """ This is the real initialization function, called when the module has been loaded """
+        self.tree      = None
         self.popup     = None
         self.currLib   = None
         self.cfgWindow = None
         self.libraries = prefs.get(__name__, 'libraries',  PREFS_DEFAULT_LIBRARIES)
         self.treeState = prefs.get(__name__, 'tree-state', PREFS_DEFAULT_TREE_STATE)
-        # Create the tree
+        # Scroll window
+        self.scrolled = gtk.ScrolledWindow()
+        self.scrolled.set_shadow_type(gtk.SHADOW_IN)
+        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.scrolled.show()
+
+
+    def isDraggable(self):
+        """ Only playable rows can be dragged """
+        for row in self.tree.iterSelectedRows():
+            if row[ROW_TYPE] != TYPE_NONE:
+                return True
+        return False
+
+
+    def __createTree(self):
+        """ Create the main tree, add it to the scrolled window """
         txtRdr    = gtk.CellRendererText()
         pixbufRdr = gtk.CellRendererPixbuf()
 
@@ -117,25 +134,13 @@ class Library(modules.Module):
         self.tree.get_column(0).set_cell_data_func(pixbufRdr, self.__drawCell)
         self.tree.setIsDraggableFunc(self.isDraggable)
         self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])
-        # Scroll window
-        self.scrolled = gtk.ScrolledWindow()
-        self.scrolled.add(self.tree)
-        self.scrolled.set_shadow_type(gtk.SHADOW_IN)
-        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.scrolled.show()
         # GTK handlers
         self.tree.connect('drag-data-get',              self.onDragDataGet)
         self.tree.connect('key-press-event',            self.onKeyPressed)
         self.tree.connect('exttreeview-row-expanded',   self.onRowExpanded)
         self.tree.connect('exttreeview-button-pressed', self.onButtonPressed)
-
-
-    def isDraggable(self):
-        """ Only playable rows can be dragged """
-        for row in self.tree.iterSelectedRows():
-            if row[ROW_TYPE] != TYPE_NONE:
-                return True
-        return False
+        # Add the tree to the scrolled window
+        self.scrolled.add(self.tree)
 
 
     def __drawCell(self, column, cell, model, iter):
@@ -178,6 +183,10 @@ class Library(modules.Module):
         newLibrary = {}                                                                # Reflect the current file structure of the library
         oldLibrary = pickleLoad(os.path.join(libPath, 'files'))                        # Previous file structure of the same library
 
+        # Make sure the root directory still exists
+        if not os.path.exists(path):
+            queue.pop()
+
         while len(queue) != 0:
             currDir      = queue.pop()
             currDirMTime = os.stat(currDir).st_mtime
@@ -267,7 +276,7 @@ class Library(modules.Module):
         overallNbArtists = len(db)
 
         # The 'artists' file contains all known artists with their index, the 'files' file contains the file structure of the root path
-        allArtists = sorted([(artist, str(indexArtist), len(db[artist])) for indexArtist, artist in enumerate(db)], key = lambda a: a[0].lower())
+        allArtists = sorted([(artist, str(indexArtist), len(db[artist])) for indexArtist, artist in enumerate(db)], key = lambda a: a[0])
         pickleSave(os.path.join(libPath, 'files'),   newLibrary)
         pickleSave(os.path.join(libPath, 'artists'), allArtists)
 
@@ -284,7 +293,7 @@ class Library(modules.Module):
                 albums.append((name, str(index), len(tracks), length))
                 pickleSave(os.path.join(artistPath, str(index)), sorted(tracks, key = lambda track: track.getNumber()))
 
-            albums.sort(key = lambda album: album[0].lower())
+            albums.sort(cmp = lambda a1, a2: cmp(db[artist][a1[0]][0], db[artist][a2[0]][0]))
             pickleSave(os.path.join(artistPath, 'albums'), albums)
             progress.pulse()
             yield True
@@ -304,10 +313,18 @@ class Library(modules.Module):
         yield False
 
 
-    def __getTracksFromSelectedRows(self, tree):
-        """ Return a list of tracks based on selected rows """
+    def __getTracksFromPaths(self, tree, paths):
+        """
+            Return the list of tracks extracted from:
+                * The list 'paths' if it is not None
+                * The currently selected rows if 'paths' is None
+        """
         tracks = []
-        for currPath in tree.getSelectedPaths():
+
+        if paths is None:
+            paths = tree.getSelectedPaths()
+
+        for currPath in paths:
             row = tree.getRow(currPath)
             if row[ROW_TYPE] == TYPE_TRACK:
                 tracks.append(row[ROW_TAGS])
@@ -331,9 +348,12 @@ class Library(modules.Module):
         return tracks
 
 
-    def playSelection(self, tree, replace):
-        """ Replace/extend the tracklist """
-        tracks = self.__getTracksFromSelectedRows(tree)
+    def playPaths(self, tree, paths, replace):
+        """
+            Replace/extend the tracklist
+            If the list 'paths' is None, use the current selection
+        """
+        tracks = self.__getTracksFromPaths(tree, paths)
 
         if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
         else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
@@ -346,8 +366,8 @@ class Library(modules.Module):
             self.popup.get_widget('menu-popup').show_all()
             # Connect handlers
             self.popup.get_widget('item-collapse').connect('activate', lambda widget: self.tree.collapse_all())
-            self.popup.get_widget('item-play').connect('activate',     lambda widget: self.playSelection(tree, True))
-            self.popup.get_widget('item-add').connect('activate',      lambda widget: self.playSelection(tree, False))
+            self.popup.get_widget('item-play').connect('activate',     lambda widget: self.playPaths(tree, None, True))
+            self.popup.get_widget('item-add').connect('activate',      lambda widget: self.playPaths(tree, None, False))
             self.popup.get_widget('item-refresh').connect('activate',  lambda widget: idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next))
 
         playable = path is not None and tree.getItem(path, ROW_TYPE) != TYPE_NONE
@@ -361,7 +381,7 @@ class Library(modules.Module):
         """ Load the given library """
         rows     = []
         path     = os.path.join(ROOT_PATH, name)
-        prevChar = None
+        prevChar = ''
 
         # Make sure the version number is the good one
         if not os.path.exists(os.path.join(path, 'VERSION_%u' % VERSION)):
@@ -372,10 +392,11 @@ class Library(modules.Module):
 
         # Create the rows, with alphabetical header if needed
         for artist in pickleLoad(os.path.join(path, 'artists')):
+
             if len(artist[ART_NAME]) != 0: currChar = unicode(artist[ART_NAME], errors='replace')[0]
             else:                          currChar = prevChar
 
-            if prevChar is None or (prevChar != currChar and not (prevChar.isdigit() and currChar.isdigit())):
+            if prevChar != currChar and not (prevChar.isdigit() and currChar.isdigit()):
                 prevChar = currChar
                 if currChar.isdigit(): rows.append((None, '<b>0 - 9</b>',         TYPE_HEADER, None, None))
                 else:                  rows.append((None, '<b>%s</b>' % currChar, TYPE_HEADER, None, None))
@@ -428,10 +449,13 @@ class Library(modules.Module):
         """ A mouse button has been pressed """
         if event.button == 3:
             self.showPopupMenu(tree, event.button, event.time, path)
-        elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS and path is not None and tree.getItem(path, ROW_TYPE) != TYPE_NONE:
-            if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.playSelection(tree, True)
-            elif tree.row_expanded(path):                         tree.collapse_row(path)
-            else:                                                 tree.expand_row(path, False)
+        elif path is not None and tree.getItem(path, ROW_TYPE) != TYPE_NONE:
+            if event.button == 2:
+                self.playPaths(tree, [path], False)
+            elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+                if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.playPaths(tree, None, True)
+                elif tree.row_expanded(path):                         tree.collapse_row(path)
+                else:                                                 tree.expand_row(path, False)
 
 
     def onKeyPressed(self, tree, event):
@@ -444,12 +468,12 @@ class Library(modules.Module):
         elif keyname == 'Right':  tree.expandRows()
         elif keyname == 'minus':  tree.collapseRows()
         elif keyname == 'space':  tree.switchRows()
-        elif keyname == 'Return': self.playSelection(tree, True)
+        elif keyname == 'Return': self.playPaths(tree, None, True)
 
 
     def onDragDataGet(self, tree, context, selection, info, time):
         """ Provide information about the data being dragged """
-        serializedTracks = '\n'.join([track.serialize() for track in self.__getTracksFromSelectedRows(tree)])
+        serializedTracks = '\n'.join([track.serialize() for track in self.__getTracksFromPaths(tree, None)])
         selection.set(consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8, serializedTracks)
 
 
@@ -475,6 +499,10 @@ class Library(modules.Module):
             idle_add(self.addAllExplorers)
 
         elif msg == consts.MSG_EVT_EXPLORER_CHANGED and params['modName'] == MOD_L10N and params['expName'] != self.currLib:
+            # Create the tree if needed
+            if self.tree is None:
+                self.__createTree()
+
             # Save the state of the current library
             if self.currLib is not None:
                 self.treeState[self.currLib] = self.tree.saveState(ROW_NAME)
@@ -488,8 +516,10 @@ class Library(modules.Module):
                 self.tree.restoreState(self.treeState[self.currLib], ROW_NAME)
 
         elif msg == consts.MSG_EVT_APP_QUIT or msg == consts.MSG_EVT_MOD_UNLOADED:
-            self.treeState[self.currLib] = self.tree.saveState(ROW_NAME)
-            prefs.set(__name__, 'tree-state', self.treeState)
+            if self.currLib is not None:
+                self.treeState[self.currLib] = self.tree.saveState(ROW_NAME)
+                prefs.set(__name__, 'tree-state', self.treeState)
+
             prefs.set(__name__, 'libraries',  self.libraries)
             self.removeAllExplorers()
 
@@ -504,10 +534,10 @@ class Library(modules.Module):
             # Create the list of libraries
             txtRdr  = gtk.CellRendererText()
             pixRdr  = gtk.CellRendererPixbuf()
-            columns = ((None, [(txtRdr, TYPE_STRING)],                           0, False),
-                       ('',   [(pixRdr, gtk.gdk.Pixbuf), (txtRdr, TYPE_STRING)], 2, False))
+            columns = ((None, [(txtRdr, TYPE_STRING)],                           0, False, False),
+                       ('',   [(pixRdr, gtk.gdk.Pixbuf), (txtRdr, TYPE_STRING)], 2, False, True))
 
-            self.cfgList = extListview.ExtListView(columns, sortable=False, useMarkup=True)
+            self.cfgList = extListview.ExtListView(columns, sortable=False, useMarkup=True, canShowHideColumns=False)
             self.cfgList.set_headers_visible(False)
             self.cfgWindow.getWidget('scrolledwindow1').add(self.cfgList)
             # Connect handlers
