@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
 
 import modules, tools
 
-from tools   import consts, prefs
-from gettext import gettext as _
+from tools   import consts,  prefs
+from gettext import ngettext, gettext  as _
 
 MOD_INFO = ('Status and Title Bars', 'Status and Title Bars', '', [], True, False)
 
@@ -46,9 +46,11 @@ class StatusbarTitlebar(modules.Module):
 
     def onNewTracklist(self, count, playtime):
         """ A new tracklist has been set """
-        if   count == 0: text = ''
-        elif count == 1: text = '%u %s  [%s]' % (count, _('track in playlist'),  tools.sec2str(playtime))
-        else:            text = '%u %s  [%s]' % (count, _('tracks in playlist'), tools.sec2str(playtime))
+        if count == 0:
+            text = ''
+        else:
+            text = ngettext('One track in playlist  [%(length)s]', '%(count)u tracks in playlist  [%(length)s]', count) \
+                    % {'count': count, 'length': tools.sec2str(playtime)}
 
         self.statusbar.pop(self.contextId)
         self.statusbar.push(self.contextId, text)
