@@ -29,8 +29,19 @@ MOD_INFO = ('Tracklist', 'Tracklist', '', [], True, False)
 # Default preferences
 PREFS_DEFAULT_REPEAT_STATUS = False
 
+PREFS_DEFAULT_COLUMNS_VISIBILITY = {
+            '#'         : True,
+            _('Title')  : True,
+            _('Artist') : True,
+            _('Album')  : True,
+            _('Date')   : False,
+            _('Genre')  : False,
+            _('Length') : True,
+            _('Path')   : False,
+        }
 
-TRACKLIST_NB_FIELDS = 7
+
+TRACKLIST_NB_FIELDS = 10
 
 (
     ROW_ICO,   # Icon drawn in front of the row
@@ -39,6 +50,9 @@ TRACKLIST_NB_FIELDS = 7
     ROW_ART,   # Track artist
     ROW_ALB,   # Track album
     ROW_LEN,   # Track length in seconds
+    ROW_GNR,   # Genre
+    ROW_DAT,   # Date
+    ROW_PTH,   # Path to the file
     ROW_TRK    # The Track object
 ) = range(TRACKLIST_NB_FIELDS)
 
@@ -74,22 +88,28 @@ class Tracklist(modules.Module):
         pixbufRdr = gtk.CellRendererPixbuf()
         txtRRdr.set_property('xalign', 1.0)
 
-        columns = (('#',         [(pixbufRdr, gtk.gdk.Pixbuf), (txtRRdr, TYPE_INT)], (ROW_NUM, ROW_TIT),                   False),
-                   (_('Title'),  [(txtLRdr, TYPE_STRING)],                           (ROW_TIT,),                           True),
-                   (_('Artist'), [(txtLRdr, TYPE_STRING)],                           (ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), True),
-                   (_('Album'),  [(txtLRdr, TYPE_STRING)],                           (ROW_ALB, ROW_NUM, ROW_TIT),          True),
-                   (_('Length'), [(txtRRdr, TYPE_INT)],                              (ROW_LEN,),                           False),
-                   (None,        [(None, TYPE_PYOBJECT)],                            (None,),                              False))
+        visible = tools.prefs.get(__name__, 'columns-visibility', PREFS_DEFAULT_COLUMNS_VISIBILITY)
 
-        self.list = ExtListView(columns, True, consts.DND_TARGETS.values())
+        columns = (('#',         [(pixbufRdr, gtk.gdk.Pixbuf), (txtRRdr, TYPE_INT)], (ROW_NUM, ROW_TIT),                            False, visible['#']),
+                   (_('Title'),  [(txtLRdr, TYPE_STRING)],                           (ROW_TIT,),                                    True,  visible[_('Title')]),
+                   (_('Artist'), [(txtLRdr, TYPE_STRING)],                           (ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT),          True,  visible[_('Artist')]),
+                   (_('Album'),  [(txtLRdr, TYPE_STRING)],                           (ROW_ALB, ROW_NUM, ROW_TIT),                   True,  visible[_('Album')]),
+                   (_('Length'), [(txtRRdr, TYPE_INT)],                              (ROW_LEN,),                                    False, visible[_('Length')]),
+                   (_('Genre'),  [(txtLRdr, TYPE_STRING)],                           (ROW_GNR, ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), False, visible[_('Genre')]),
+                   (_('Date'),   [(txtLRdr, TYPE_INT)],                              (ROW_DAT, ROW_ART, ROW_ALB, ROW_NUM, ROW_TIT), False, visible[_('Date')]),
+                   (_('Path'),   [(txtLRdr, TYPE_STRING)],                           (ROW_PTH,),                                    False, visible[_('Path')]),
+                   (None,        [(None, TYPE_PYOBJECT)],                            (None,),                                       False, False))
+
+        self.list = ExtListView(columns, sortable=True, dndTargets=consts.DND_TARGETS.values(), useMarkup=False, canShowHideColumns=True)
         self.list.get_column(4).set_cell_data_func(txtRRdr, self.fmtLength)
         self.list.enableDNDReordering()
         wTree.get_widget('scrolledwindow1').add(self.list)
         # GTK handlers
-        self.list.connect('extlistview-dnd',            self.onDND)
-        self.list.connect('key-press-event',            self.onKeyboard)
-        self.list.connect('extlistview-modified',       self.onListModified)
-        self.list.connect('extlistview-button-pressed', self.onButtonPressed)
+        self.list.connect('extlistview-dnd',                       self.onDND)
+        self.list.connect('key-press-event',                       self.onKeyboard)
+        self.list.connect('extlistview-modified',                  self.onListModified)
+        self.list.connect('extlistview-button-pressed',            self.onButtonPressed)
+        self.list.connect('extlistview-column-visibility-changed', self.onColumnVisibilityChanged)
 
         self.btnClear.connect('clicked',   lambda widget: modules.postMsg(consts.MSG_CMD_TRACKLIST_CLR))
         self.btnRepeat.connect('toggled',  self.onButtonRepeat)
@@ -163,8 +183,8 @@ class Tracklist(modules.Module):
 
     def insert(self, tracks, position=None):
         """ Insert some tracks in the tracklist, append them if position is None """
-        rows = [[consts.icoNull, track.getNumber(), track.getTitle(), track.getArtist(), track.getExtendedAlbum(), track.getLength(), track]
-                    for track in tracks]
+        rows = [[consts.icoNull, track.getNumber(), track.getTitle(), track.getArtist(), track.getExtendedAlbum(),
+                    track.getLength(), track.getGenre(), track.getDate(), track.getURI(), track] for track in tracks]
 
         for row in rows:
             self.playtime += row[ROW_LEN]
@@ -313,6 +333,13 @@ class Tracklist(modules.Module):
         modules.postMsg(consts.MSG_EVT_TRACK_MOVED,   {'hasPrevious': self.hasPrev(), 'hasNext':  self.hasNext()})
 
 
+    def onColumnVisibilityChanged(self, list, colTitle, visible):
+        """ A column has been shown/hidden """
+        visibility = tools.prefs.get(__name__, 'columns-visibility', PREFS_DEFAULT_COLUMNS_VISIBILITY)
+        visibility[colTitle] = visible
+        tools.prefs.set(__name__, 'columns-visibility', visibility)
+
+
     def onDND(self, list, context, x, y, dragData, dndId, time):
         """ External Drag'n'Drop """
         if dragData.data == '':
@@ -321,9 +348,12 @@ class Tracklist(modules.Module):
 
         dropInfo = list.get_dest_row_at_pos(x, y)
 
-        # A list of filenames
-        if dndId == consts.DND_DAP_URI or dndId == consts.DND_URI:
+        # A list of filenames, without 'file://' at the beginning
+        if dndId == consts.DND_DAP_URI:
             tracks = media.getTracks([urllib.url2pathname(uri) for uri in dragData.data.split()])
+        # A list of filenames starting with 'file://'
+        elif dndId == consts.DND_URI:
+            tracks = media.getTracks([urllib.url2pathname(uri)[7:] for uri in dragData.data.split()])
         # A list of tracks
         elif dndId == consts.DND_DAP_TRACKS:
             tracks = [media.track.unserialize(serialTrack) for serialTrack in dragData.data.split('\n')]
