@@ -33,7 +33,7 @@ socketTimeout = 10
 
 # --- Strings
 appName      = 'Decibel Audio Player'
-appVersion   = '0.11'
+appVersion   = '1.00'
 appNameShort = 'decibel-audio-player'
 
 
@@ -169,7 +169,7 @@ DND_TARGETS = {
     MSG_CMD_EXPLORER_REMOVE,   # Remove an explorer    Parameters: 'modName', 'expName'
 
     # Covers
-    MSG_CMD_SET_COVER,         # Cover file for the given track     Parameters: 'track', 'path'
+    MSG_CMD_SET_COVER,         # Cover file for the given track     Parameters: 'track', 'pathThumbnail', 'pathFullSize'
 
     # --== EVENTS ==--
 
